@@ -36,24 +36,23 @@ _get_ves()
 
 _vzctl()
 {
-	local cur prev cmd vzctl_cmds vzctl_create_opts vzctl_set_opts
-	local iptables_names cap_names
-
 	#echo "ARGS: $*"
 	#echo "COMP_WORDS: $COMP_WORDS"
 	#echo "COMP_CWORD: $COMP_CWORD"
 	#echo "COMP_WORDS[1]: ${COMP_WORDS[1]}"
 
 	COMPREPLY=()
-	cur=${COMP_WORDS[COMP_CWORD]}
-	prev=${COMP_WORDS[COMP_CWORD-1]}
+	local cur=${COMP_WORDS[COMP_CWORD]}
+	local prev=${COMP_WORDS[COMP_CWORD-1]}
 
-	vzctl_common_opts="--quiet --verbose --help --version"
-	vzctl_cmds="create destroy delete mount umount chkpnt restore \
-		set start stop restart status enter exec exec2 runscript"
-	vzctl_create_opts="--ostemplate --config --private --root \
+
+	local vzctl_common_opts="--quiet --verbose --help --version"
+	local vzctl_cmds="create destroy delete mount umount chkpnt restore \
+		set start stop restart status quotaon quotaoff quotainit \
+		enter exec exec2 runscript"
+	local vzctl_create_opts="--ostemplate --config --private --root \
 		--ipadd --hostname --name --description"
-	vzctl_set_opts="--save --onboot --root --private \
+	local vzctl_set_opts="--save --onboot --root --private \
 		--userpasswd --disabled --ipadd --ipdel \
 		--hostname --nameserver --searchdomain \
 		--numproc --numtcpsock --numothersock --vmguarpages \
@@ -63,27 +62,28 @@ _vzctl()
 		--dcachesize --numiptent --physpages --swappages \
 		--cpuunits --cpulimit --cpus \
 		--iptables --netdev_add --netdev_del --netif_add --netif_del \
+		--diskquota \
 		--diskspace --diskinodes --quotatime --quotaugidlimit \
 		--noatime --capability --devnodes --devices --applyconfig \
 		--name --ioprio --setmode --meminfo --bootorder --features"
-	vzctl_start_opts="--wait --force"
-	vzctl_stop_opts="--fast"
-	vzctl_restart_opts="$vzctl_start_opts $vzctl_stop_opts"
+	local vzctl_start_opts="--wait --force"
+	local vzctl_stop_opts="--fast"
+	local vzctl_restart_opts="$vzctl_start_opts $vzctl_stop_opts"
 
-	iptables_names="iptable_filter iptable_mangle ipt_limit
+	local iptables_names="iptable_filter iptable_mangle ipt_limit
 		ipt_multiport ipt_tos ipt_TOS ipt_REJECT ipt_TCPMSS
 		ipt_tcpmss ipt_ttl ipt_LOG ipt_length ip_conntrack
 		ip_conntrack_ftp ip_conntrack_irc ipt_conntrack ipt_state
 		ipt_helper iptable_nat ip_nat_ftp ip_nat_irc ipt_REDIRECT"
 
-	cap_names="chown dac_override dac_read_search fowner fsetid kill
+	local cap_names="chown dac_override dac_read_search fowner fsetid kill
 		setgid setuid setpcap linux_immutable net_bind_service
 		net_broadcast net_admin net_raw ipc_lock ipc_owner
 		sys_module sys_rawio sys_chroot sys_ptrace sys_pacct
 		sys_admin sys_boot sys_nice sys_resource sys_time
 		sys_tty_config mknod lease setveid ve_admin"
 
-	feature_names="sysfs nfs sit ipip ppp ipgre bridge nfsd"
+	local feature_names="sysfs nfs sit ipip ppp ipgre bridge nfsd"
 
 
 	case $COMP_CWORD in
@@ -109,8 +109,8 @@ _vzctl()
 		;;
 
 	*) # COMP_CWORD >= 3
-		if [[ $COMP_CWORD -eq 3 && ! $prev != '^[1-9][0-9]*$' ]]; then
-			# CTID
+		if [ $COMP_CWORD -eq 3 -a ${COMP_WORDS[1]::2} = "--" ]; then
+			# Global option was used, need CTID here
 			COMPREPLY=( $( compgen -W "$(_get_ves $prev)" -- $cur ) )
 		else
 
@@ -123,7 +123,7 @@ _vzctl()
 						    sed -e "s#^$vztmpl/cache/##" -e 's#.tar.gz$##')" -- $cur ) )
 
 				;;
-			--onboot|--disabled|--noatime)
+			--onboot|--disabled|--noatime|--diskquota)
 				COMPREPLY=( $( compgen -W "yes no" -- $cur ) )
 				;;
 			--setmode)
@@ -133,7 +133,6 @@ _vzctl()
 				local configs=$(command ls -1 /etc/vz/conf/*.conf-sample |
 						    cut -d "-" -f 2- | sed -e 's#.conf-sample$##')
 
-				configs=${configs/.conf-sample/}
 				COMPREPLY=( $( compgen -W "$configs" -- $cur ) )
 				;;
 			--iptables)
@@ -189,21 +188,22 @@ _vzctl()
 				# when user has root privileges
 				[ "$EUID" -ne 0 ] && exit 1
 				# Get CTID
+				local cmd=${COMP_WORDS[1]}
 				local ve=${COMP_WORDS[2]}
-				if [[ ! ${ve} != '^[1-9][0-9]*$' ]] ; then
+				if [ ${cmd::2} = "--" ] ; then
 					# --verbose or --quiet used
 					ve=${COMP_WORDS[3]}
 				fi
-				# NAME or ID ?
-				LIST_OPT=`echo $ve | awk '/[a-zA-Z]/ {print "-N"}'`
-				local ips="`/usr/sbin/vzlist -H -o ip $LIST_OPT $ve 2>/dev/null | grep -vi -`"
+				local ips="`/usr/sbin/vzlist -H -o ip $ve 2>/dev/null | grep -vi -`"
 				COMPREPLY=( $( compgen -W "$ips all" -- $cur ) )
 				;;
 			--netif_del)
 				local netif
 				if [ -f /proc/vz/veth ]; then
+					# Get CTID
+					local cmd=${COMP_WORDS[1]}
 					local ve=${COMP_WORDS[2]}
-					if [[ ! ${ve} != '^[1-9][0-9]*$' ]] ; then
+					if [ ${cmd::2} = "--" ] ; then
 						# --verbose or --quiet used
 						ve=${COMP_WORDS[3]}
 					fi
@@ -226,7 +226,7 @@ _vzctl()
 			*)
 				if [[ "${prev::2}" != "--" || "$prev" = "--save" ]]; then
 					# List options
-					cmd=${COMP_WORDS[1]}
+					local cmd=${COMP_WORDS[1]}
 					if [ ${cmd::2} = "--" ] ; then
 						# --verbose or --quiet used
 						cmd=${COMP_WORDS[2]}
